@@ -2,13 +2,14 @@ package Text::Reform;
 
 use strict; use vars qw($VERSION @ISA @EXPORT @EXPORT_OK); use Carp;
 use 5.005;
-$VERSION = '1.10';
+#use version;
+$VERSION = '1.20';
 
 require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw( form );
-@EXPORT_OK = qw( tag break_with break_wrap break_TeX );
+@EXPORT_OK = qw( columns tag break_with break_at break_wrap break_TeX debug );
 
 my @bspecials = qw( [ | ] );
 my @lspecials = qw( < ^ > );
@@ -24,9 +25,8 @@ my $emptyref = '';
 
 sub import
 {
-	use POSIX qw( localeconv );
-	$DB::single=1;
-	$decimal = localeconv()->{decimal_point} || '.';
+	#$decimal = localeconv()->{decimal_point} || '.';
+	$decimal = '.';
 
 	my $lnumerical = '[>]+(?:'.quotemeta($decimal).'[<]{1,})';
 	my $bnumerical = '[]]+(?:'.quotemeta($decimal).'[[]{1,})';
@@ -44,7 +44,7 @@ sub import
 }
 
 sub carpfirst {
-	our %carped;
+	use vars '%carped';
 	my ($msg) = @_;
 	return if $carped{$msg}++;
 	carp $msg;
@@ -69,8 +69,8 @@ sub break_with
 		}
 		else
 		{
-			@ret = (substr($_[0],0,$_[1]-length($hyphen)),
-				substr($_[0],$_[1]-length($hyphen)))
+			@ret = (substr($_[0],0,$_[1]-$hylen),
+				substr($_[0],$_[1]-$hylen))
 		}
 		if ($ret[0] =~ /\A\s*\Z/) { return ("",$_[0]); }
 		else { return ($ret[0].$hyphen,$ret[1]); }
@@ -78,6 +78,38 @@ sub break_with
 
 }
 
+sub break_at {
+	my ($hyphen, $opts_ref) = @_;
+	my $hylen = length($hyphen);
+	my $except = $opts_ref->{except};
+	my @ret;
+	sub
+	{
+		my $max = $_[2]-$hylen;
+		if ($max <= 0) {
+			@ret = (substr($_[0],0,1), substr($_[0],1))
+		}
+        elsif (defined $except && $_[0] =~ m/\A (.{1,$max}) ($except .*)/xms) {
+			@ret = ($1,$2);
+        }
+        elsif (defined $except && $_[0] =~ m/\A ($except) (.*)/xms) {
+			@ret = ($1,$2);
+        }
+		elsif ($_[0] =~ /\A (.{1,$max}$hyphen) (.*)/xms) {
+			@ret = ($1,$2);
+		}
+		elsif (length($_[0])>$_[2]) {
+			@ret = (substr($_[0],0,$_[1]-$hylen).$hyphen,
+				substr($_[0],$_[1]-$hylen))
+		}
+		else {
+			@ret = ("",$_[0]);
+		}
+		if ($ret[0] =~ /\A\s*\Z/) { return ("",$_[0]); }
+		else { return @ret; }
+	}
+}
+
 sub break_wrap
 {
 	return \&break_wrap unless @_;
@@ -91,7 +123,7 @@ sub break_TeX
 {
 	my $file = $_[0] || "";
 
-	croak "Can't find TeX::Hypen module"
+	croak "Can't find TeX::Hyphen module"
 		unless require "TeX/Hyphen.pm";
 
 	$hyp{$file} = TeX::Hyphen->new($file||undef)
@@ -109,15 +141,28 @@ sub break_TeX
 	}
 }
 
-sub debug { print STDERR @_, "\n" if $::DEBUG || $::DEBUG }
+my $debug = 0;
+sub _debug { print STDERR @_, "\n" if $debug }
+sub debug { $debug = 1; }
 
 sub notempty
 {
 	my $ne = ${$_[0]} =~ /\S/;
-	debug("\tnotempty('${$_[0]}') = $ne\n");
+	_debug("\tnotempty('${$_[0]}') = $ne\n");
 	return $ne;
 }
 
+sub strtod1 {
+  my $n=shift;
+  my $real_re='((?:[+-]?)(?:(?=[0123456789]|[.])(?:[0123456789]*)(?:(?:[.])(?:[0123456789]{0,}))?)'.
+  '(?:(?:[eE])(?:(?:[+-]?)(?:[0123456789]+))|))';
+  if ($n=~/^\s*$real_re(.*)$/os) {
+    return ($1,length($2 || ''));
+  } else {
+    return (undef,length($n));
+  }
+}
+
 sub replace($$$$)   # ($fmt, $len, $argref, $config)
 {
 	my $ref = $_[2];
@@ -134,9 +179,9 @@ sub replace($$$$)   # ($fmt, $len, $argref, $config)
 	if ($$ref =~ /\S/ && $fmtnum>2)
 	{
 	NUMERICAL:{
-		use POSIX qw( strtod );
 		my ($ilen,$dlen) = map {length} $_[0] =~ m/([]>]+)\Q$decimal\E([[<]+)/;
-		my ($num,$unconsumed) = strtod($$ref);
+		my ($num,$unconsumed) = strtod1($$ref);
+
 		if ($unconsumed == length $$ref)
 		{
 			$$ref =~ s/\s*\S*//;
@@ -176,21 +221,27 @@ sub replace($$$$)   # ($fmt, $len, $argref, $config)
 			$ws =~ s/\n/$nonnl? "" : " "/ge if $config->{fill};
 			my $lead = ($config->{squeeze} ? ($ws ? " " : "") : $ws);
 			my $match = $lead . $word;
+			_debug "Extracted [$match]";
 			last if $text && $match =~ /\n/;
 			my $len1 = length($match);
 			if ($len1 <= $rem)
 			{
+				_debug "Accepted [$match]";
 				$text .= $match;
 				$rem  -= $len1;
 				$$ref = $extra;
 			}
 			else
 			{
-				if ($len1 > $_[1] and $rem-length($lead)>$config->{minbreak})
+				_debug "Need to break [$match]";
+				# was: if ($len1 > $_[1] and $rem-length($lead)>$config->{minbreak})
+				if ($rem-length($lead)>$config->{minbreak})
 				{
+					_debug "Trying to break '$match'";
 					my ($broken,$left) =
 						$config->{break}->($match,$rem,$_[1]);	
 					$text .= $broken;
+					_debug "Broke as: [$broken][$left]";
 					$$ref = $left.$extra;
 					$rem -= length $broken;
 				}
@@ -324,7 +375,7 @@ sub fix_config(\%)
 	unless (ref $config->{pagefeed} eq 'CODE')
 		{ my $tmp = $config->{pagefeed}; $config->{pagefeed} = sub { $tmp } }
 	unless (ref $config->{break} eq 'CODE')
-		{ $config->{break} = break_with($config->{break}) }
+		{ $config->{break} = break_at($config->{break}) }
 	if (defined $config->{pagenum} && ref $config->{pagenum} ne 'SCALAR') 
 		{ my $tmp = $config->{pagenum}+0; $config->{pagenum} = \$tmp }
 	unless (ref $config->{filler} eq 'HASH') {
@@ -342,7 +393,7 @@ sub FormOpt::DESTROY
 
 sub form
 {
-	our %carped;
+	use vars '%carped';
 	local %carped;
 	my $config = {%std_config};
 	my $startidx = 0;
@@ -407,10 +458,10 @@ sub form
 		}
 		elsif (!defined eval { local $SIG{__DIE__};
 				       $_[$nextarg] = $next;
-				       debug "writeable: [$_[$nextarg]]";
+				       _debug "writeable: [$_[$nextarg]]";
 				       1})
 		{
-		        debug "unwriteable: [$_[$nextarg]]";
+		        _debug "unwriteable: [$_[$nextarg]]";
 			my $arg = $_[$nextarg];
 			splice @_, $nextarg, 1, \$arg;
 		}
@@ -451,7 +502,7 @@ sub form
 				: ${$_[$startidx++]}||"";
 		}
 		my $format = shift @format_stack;
-		debug("format: [$format]");
+		_debug("format: [$format]");
 	
 		my @parts = split /(\n|(?:\\.)+|$fieldpat)/, $format;
 		push @parts, "\n" unless @parts && $parts[-1] eq "\n";
@@ -466,7 +517,7 @@ sub form
 			{
 				if ($part =~ /\A(?:\\.)+/)
 				{
-					debug("esc literal: [$part]");
+					_debug("esc literal: [$part]");
 					my $tmp = $part;
 					$tmp =~ s/\\(.)/$1/g;
 					$text .= $tmp;
@@ -481,15 +532,15 @@ sub form
 						my $type = $1;
 						$type = 'J' if $part =~ /$ljustified/;
 						croak BAD_CONFIG if ($ref[$startidx] eq 'HASH');
-						debug("once field: [$part]");
-						debug("data was: [${$_[$nextarg]}]");
+						_debug("once field: [$part]");
+						_debug("data was: [${$_[$nextarg]}]");
 						$text .= replace($type,length($part),$_[$nextarg],$config);
-						debug("data now: [${$_[$nextarg]}]");
+						_debug("data now: [${$_[$nextarg]}]");
 					}
 					else
 					{
 						$text .= substr($config->{filler}{left} x length($part), -length($part));
-						debug("missing once field: [$part]");
+						_debug("missing once field: [$part]");
 					}
 					$nextarg++;
 				}
@@ -501,16 +552,16 @@ sub form
 					my $type = $1;
 					$type = 'J' if $part =~ /$bjustified/;
 					croak BAD_CONFIG if ($ref[$startidx] eq 'HASH');
-					debug("multi field: [$part]");
-					debug("data was: [${$_[$nextarg]}]");
+					_debug("multi field: [$part]");
+					_debug("data was: [${$_[$nextarg]}]");
 					$text .= replace($type,length($part),$_[$nextarg],$config);
-					debug("data now: [${$_[$nextarg]}]");
+					_debug("data now: [${$_[$nextarg]}]");
 					push @data, $_[$nextarg];
 					$nextarg++;
 				}
 				else
 				{
-					debug("literal: [$part]");
+					_debug("literal: [$part]");
 					my $tmp = $part;
 					$tmp =~ s/\0(\0*)/$1/g;
 					$text .= $tmp;
@@ -519,7 +570,7 @@ sub form
 						$linecount++;
 						if ($config->{pagelen} && $linecount>=$config->{pagelen})
 						{
-							debug("\tejecting page:  $config->{pagenum}");
+							_debug("\tejecting page:  $config->{pagenum}");
 							carpfirst "\nWarning: could not format page ${$config->{pagenum}} within specified page length"
 								if $^W && $config->{pagelen} && $linecount > $config->{pagelen};
 							${$config->{pagenum}}++;
@@ -539,8 +590,8 @@ sub form
 						}
 					}
 				}
-				debug("\tnextarg now:  $nextarg");
-				debug("\tstartidx now: $startidx");
+				_debug("\tnextarg now:  $nextarg");
+				_debug("\tstartidx now: $startidx");
 			}
 			$firstline = 0;
 			$filled = ! grep { notempty $_ } @data;
@@ -604,7 +655,7 @@ sub form
                 if ($ref[$i] eq 'ARRAY')
                         { eval { @{$orig[$i]} = map "$_\n", split /\n/, ${$_[$i]} } }
                 elsif (!$ref[$i])
-                        { eval { debug("restoring $i (".$_[$i].") to " .
+                        { eval { _debug("restoring $i (".$_[$i].") to " .
                                  defined($orig[$i]) ? $orig[$i] : "<undef>");
                                  ${$_[$i]} = $orig[$i] } }
         }
@@ -693,8 +744,8 @@ Text::Reform - Manual text wrapping and reformatting
 
 =head1 VERSION
 
-This document describes version 1.10 of Text::Reform,
-released April  9, 2003.
+This document describes version 1.20 of Text::Reform,
+released 2009-09-06.
 
 =head1 SYNOPSIS
 
@@ -736,7 +787,7 @@ A single < is formatted as the literal character '<'
 Right-justified field indicator.
 A series of two or more sequential >'s specify
 a right-justified field to be filled by a subsequent value.
-A single < is formatted as the literal character '<'
+A single > is formatted as the literal character '>'
 
 =item <<<>>>
 
@@ -749,7 +800,7 @@ must be at least 2 '<' and 2 '>'.
 Centre-justified field indicator.
 A series of two or more sequential ^'s specify
 a centred field to be filled by a subsequent value.
-A single ^ is formatted as the literal character '<'
+A single ^ is formatted as the literal character '^'
 
 =item >>>.<<<<
 
@@ -960,7 +1011,7 @@ look-up table:
         \@values, \@squares, \@roots, \@logs, \@inverses;
 
 The multiline format specifier:
-        
+
         "| [[  |  [[[  |  [[[[[[[[[[ | [[[[[[[[[ | [[[[[[[[[ |
         -----------------------------------------------------",
 
@@ -1161,15 +1212,62 @@ which could therefore be rewritten:
              @data;
 
 The subroutine C<Text::Reform::break_with> takes a single string
-argument and returns a reference to a sub which hyphenates with that
-string. Hence the first of the two examples could be rewritten:
+argument and returns a reference to a sub which hyphenates by cutting 
+off the text at the right margin and appending the string argument.
+Hence the first of the two examples could be rewritten:
 
-        use Text::Reform qw( form break_wrap );
+        use Text::Reform qw( form break_with );
 
         form { break => break_with('~') }
              $format_str,
              @data;
 
+The subroutine C<Text::Reform::break_at> takes a single string
+argument and returns a reference to a sub which hyphenates by
+breaking immediately after that string. For example:
+
+        use Text::Reform qw( form break_at );
+
+        form { break => break_at('-') }
+               "[[[[[[[[[[[[[[",
+	       "The Newton-Raphson methodology";
+
+	# returns:
+	#
+	#       "The Newton-
+	#        Raphson 
+	#        methodology"
+
+Note that this differs from the behaviour of C<break_with>, which
+would be:
+
+        form { break => break_with('-') }
+               "[[[[[[[[[[[[[[",
+	       "The Newton-Raphson methodology";
+
+	# returns:
+	#
+	#       "The Newton-R-
+	#        aphson metho-
+	#        dology"
+
+Hence C<break_at> is generally a better choice.
+
+C<break_at> also takes an 'except' option, which tells the resulting
+subroutine not to break in the middle of certain strings. For example:
+
+        form { break => break_at('-', {except=>qr/Newton-Raphson/}) }
+               "[[[[[[[[[[[[[[",
+	       "The Newton-Raphson methodology";
+
+	# returns:
+	#
+	#       "The
+    #        Newton-Raphson 
+	#        methodology"
+
+This option is particularly useful for preserving URLs.
+
 The subroutine C<Text::Reform::break_TeX> 
 returns a reference to a sub which hyphenates using 
 Jan Pazdziora's TeX::Hyphen module. For example:
@@ -1354,7 +1452,7 @@ numeric values about a fixed decimal place marker. For example:
         EONUMS
 
 would print:
-                   
+
         (    1.0 )
         (    1.0 )
         (    1.00)
@@ -1380,7 +1478,7 @@ by giving the configuration option 'numeric' a value that matches
         EONUMS
 
 would print:
-                   
+
         (    1.00)
         (    1.00)
 
@@ -1405,7 +1503,7 @@ ignored. For example:
         EONUMS
 
 would print:
-                   
+
         (    1.0 )
         (    4.0 )
 
@@ -1786,12 +1884,40 @@ Damian Conway (damian@conway.org)
 
 =head1 BUGS
 
+The module uses C<POSIX::strtod>, which may be broken under certain versions
+of Windows. Applying the WINDOWS_PATCH patch to Reform.pm will replace the
+POSIX function with a copycat subroutine.
+
 There are undoubtedly serious bugs lurking somewhere in code this funky :-)
 Bug reports and other feedback are most welcome.
 
-=head1 COPYRIGHT
+=head1 LICENCE AND COPYRIGHT
 
-Copyright (c) 1997-2000, Damian Conway. All Rights Reserved.
-This module is free software. It may be used, redistributed
-and/or modified under the terms of the Perl Artistic License
-  (see http://www.perl.com/perl/misc/Artistic.html)
+Copyright (c) 1997-2007, Damian Conway C<< <DCONWAY@CPAN.org> >>. All rights reserved.
+
+This module is free software; you can redistribute it and/or
+modify it under the same terms as Perl itself. See L<perlartistic>.
+
+
+=head1 DISCLAIMER OF WARRANTY
+
+BECAUSE THIS SOFTWARE IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
+FOR THE SOFTWARE, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN
+OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
+PROVIDE THE SOFTWARE "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
+EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
+ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOFTWARE IS WITH
+YOU. SHOULD THE SOFTWARE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL
+NECESSARY SERVICING, REPAIR, OR CORRECTION.
+
+IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
+WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR
+REDISTRIBUTE THE SOFTWARE AS PERMITTED BY THE ABOVE LICENCE, BE
+LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL,
+OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE
+THE SOFTWARE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA BEING
+RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A
+FAILURE OF THE SOFTWARE TO OPERATE WITH ANY OTHER SOFTWARE), EVEN IF
+SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
+SUCH DAMAGES.
